/*
** GPL Licensed, by: Henrik, Wladimir, rue
*/


// policy service (called by refilterNode) -- global-var.
var policy = Components.classes["@mozilla.org/layout/content-policy;1"].getService(Components.interfaces.nsIContentPolicy);
var appShell = Components.classes["@mozilla.org/appshell/appShellService;1"].getService(Components.interfaces.nsIAppShellService);
var hiddenWnd = appShell.hiddenDOMWindow; // global hidden-window
var apiConstants = hiddenWnd.apiConstants;

// user clicked "OK", so save the pattern and 'live-filter' the target item
function doOK(item) {
	var textbox = document.getElementById("tbox");
		
	if (adblockIsRegExp(textbox.value, true)) {
		if (!adblockWarnRegExp()) // if the user didn't want to enter a regular expression, cancel the "OK"
			return false; }
  	
	addFilter(textbox.value); // save the filter to prefs
	item.ownerDocument.defaultView.top._AdblockFiltered = []; // reset filter-cache
	refilterFrameCheck(item.ownerDocument.defaultView.top); // refilter page

	return true;
}

// adds the filter to prefs
function addFilter(filter) {
	if (filter != "") {
		try {
			var prefObj = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
			var Branch = prefObj.getBranch("adblock.");
			var wPatterns = Branch.prefHasUserValue("patterns") ? Branch.getCharPref("patterns") : null;
		} catch(e) {}
		
		if (wPatterns) wPatterns += " " + filter;
		else wPatterns = filter;
		try {
			Branch.setCharPref("patterns", wPatterns);
			//prefObj.savePrefFile(null); // save the prefs to disk -- component.js "loadSettings()" handles this now
			var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
			observerService.notifyObservers(null, "Adblock-PrefChange", "FilterChange");
			observerService.notifyObservers(null, "Adblock-SavePrefFile", null);
		} catch(e) {}
	}			
}

// user clicked "Cancel", so close out
function doCancel() {  	
	return true;
}

// refilters all nodes of a window
function refilterFrameCheck(wnd) {
	traverseChildren(wnd.document, wnd);

	for (var i=0; i<wnd.frames.length; i++)
		refilterFrameCheck(wnd.frames[i]);
}

function traverseChildren(node, wnd) {
	for (var child = node.firstChild; child; child = child.nextSibling)
	{
		if (typeof(child._AdblockData) != 'undefined')
			refilterNode(child, wnd);

		if (child.firstChild)
			traverseChildren(child, wnd);
	}
}

// checks if a node should be displayed with the changed filters
function refilterNode(node, wnd) {
	var data = node._AdblockData; // for applets, the node will be their frame, so we can do this
	var appletFrame = null;
	if (typeof(node.contentWindow) != 'undefined' && node.contentWindow)
		wnd = node.contentWindow;
	if (typeof(node._AdblockObject) != 'undefined') {
		appletFrame = node; // special handling for applet-frames
		node = node._AdblockObject; } // special handling for applets

	for (var i=0; i<data.length; i+=3)
		if (!data[i+2]) { // if we don't have a filter-match already,
			if (typeof(node._AdblockData) != 'undefined') 
				delete node._AdblockData; // the policy-check rewrites this
			policy.shouldLoad(data[i+1], data[i], 
					apiConstants.oldStyleAPI?node:null, 
					apiConstants.oldStyleAPI?wnd:node, 
					apiConstants.IMMEDIATE, // the "immediate" boolean directs non-delayed collapsation
					null);
			//if (!policy.shouldLoad( blah blah )) { 
				/*node.style.visibility = "hidden";
				if (appletFrame) appletFrame.style.visibility = "hidden";*/ //}
		}
}

// set the input-field to the target item's source
function init() {
	var textelement = document.getElementById("tbox");
	textelement.setAttribute("value", window.arguments[0]);
}